/**
 * Check if an object contains the desired property
 * @param {Object} o - The object
 * @param {*} k - The property
 */

const has = (o, k) => Object.prototype.hasOwnProperty.call(o, k);

/**
 * Merge objects/arrays together, also merging their keys, values and other objects together as well.
 * @param {Object|Array<Object>} merging - The objects to merge seperated by commas. The last provided object is the one all previous objects are merged into.
 * @returns {Object}
 */

function merge(...merging) {
    let object = merging.pop();

    for ( const m of merging ) {
        if (getType(m) !== 'object' && getType(m) !== 'array') throw new TypeError('Only objects and arrays can be merged.')

        for ( const k in m ) {
            switch (getType(m[k])) {
                case 'object':
                    if (getType(object[k]) !== 'object') object[k] = !object[k] ? {} : Object.assign({}, object[k]);
                    merge(m[k], object[k]);
                    break;

                case 'array':
                    if (getType(object[k]) !== 'array') object[k] = !object[k] ? [] : [object[k]];
                    for (const i in m[k]) object[k].push(m[k][i]);
                    break;

                default:
                    object[k] = m[k];
            }
        }
    }

    return object;
}

/**
 * Get the proper type of a JavaScript construct
 * @param {*} x - The js constructor to get the type of
 * @returns {string}
 */

function getType(x) {
    if (typeof x === 'string' || x instanceof String) return 'string';
    if (typeof x === 'number' && isFinite(x)) return 'number';
    if (x && typeof x === 'object' && x.constructor === Array) return 'array';
    if (typeof x === 'function') return 'function';
    if (x && typeof x === 'object' && x.constructor === Object) return 'object';
    if (x instanceof Map) return 'map';
    if (x === null) return 'null';
    if (typeof x === 'undefined') return 'undefined';
    if (typeof x === 'boolean') return 'boolean';
    if (x && typeof x === 'object' && x.constructor === RegExp) return 'regexp';
    if (x instanceof Date) return 'date';
    if (typeof x === 'symbol') return 'symbol';
    return typeof x;
}

module.exports = merge;